

RAIN(6)                   Linux Programmer's Manual                    RAIN(6)


NNNAAAMMMEEE
    rain - animated raindrops display

SSSYYYNNNOOOPPPSSSIIISSS
    rain

DDDEEESSSCCCRRRIIIPPPTTTIIIOOONNN

    _R_a_i_n's display is modeled after the VAX/VMS program of the same name.  The
    terminal has to be set for 9600 baud to obtain the proper effect.

    As with all programs that use _t_e_r_m_c_a_p, the TERM environment variable  must
    be set (and exported) to the type of the terminal being used.

FFFIIILLLEEESSS
    /etc/termcap

AAAUUUTTTHHHOOORRR
    Eric P. Scott







































4Linux                             June 23, 1990                             1

